<?php
namespace Controller;

use Service\DeleteCascadeService;
use Zeedhi\Framework\DTO;
use Service\Manager\Mongo\Manager;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Dashboard extends Crud {

	/** @var \Service\Widget $dashboardService */
	private $dashboardService;

	protected $dataSourceName = "view";

	public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, \Service\Dashboard $dashboardService) {
		parent::__construct($organizationMongoManager, $deleteCascadeService);
		$this->dashboardService = $dashboardService;
	}
	
	public function buildDashboardsByMenu(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			if(!empty($row['menuId'])) {
				$filterParams = isset($row['drillDownParameters']) ? $row['drillDownParameters'] : [];
				list($dashboards, $error) = $this->dashboardService->buildDashboardsByMenu($row['menuId'], $filterParams);
				$response->addDataSet(new DataSet('dashboards', $dashboards));
				$response->addDataSet(new DataSet('errorMessages', $error));
			} else {
				$response->setError(new DTO\Response\Error("Invalid paramters. Missing the menu id.", 500));
			}
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function buildMainDashboards(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$dashboards = $this->dashboardService->buildMainDashboards();

            $viewService = InstanceManager::getInstance()->getService('\Service\View');
            $userId = InstanceManager::getInstance()->getService('environment')->getCurrentUserId();
            $userViews = $viewService->getUserViews($userId);
            
			$response->addDataSet(new DataSet('dashboards', $dashboards));
            $response->addDataSet(new DataSet('userViews', $userViews));
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

    public function buildGenericDashboards(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$dashboards = $this->dashboardService->buildGenericDashboards();
			$response->addDataSet(new DataSet('dashboards', $dashboards));
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function buildPreviewDashboards(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$dashboards = $this->dashboardService->buildPreviewDashboards($request->getRow()['dashboards']);
			$response->addDataSet(new DataSet('dashboards', $dashboards));
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function getDefaultOrgDashboards(DTO\Request\Filter $request, DTO\Response $response){
		try {
			$filterCriteria = $request->getFilterCriteria();
			$filterCriteria->setDataSourceName($this->dataSourceName);
			$indicators = $this->dashboardService->getDataFromMultipleOrgs($filterCriteria);
			$response->addDataSet(new DataSet($this->dataSourceName, $indicators));

		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function checkDrillDownIndicators(DTO\Request\Row $request, DTO\Response $response) {
		$dashboardsIds = $request->getRow()['dashboards'];
		list($drillDownIndicators, $drillDownIndicatorsNames) = $this->dashboardService->checkDrillDownIndicators($dashboardsIds);
		$response->addDataSet(new DataSet('checkDrillDownIndicators',$drillDownIndicators));
		$response->addDataSet(new DataSet('checkDrillDownIndicatorsNames',$drillDownIndicatorsNames));
    }

	public function importDefaultOrgDashboards(DTO\Request\Row $request, DTO\Response $response){
		try {
			$row = $request->getRow();
			$dashboardsId = $row['dashboards'];
			$additionalIndicators = !empty($row['additionalIndicators']) ? $row['additionalIndicators'] : null;
			if(!empty($dashboardsId)) {
				list($importedItems,$indicatorImported,$dataSourceImported) = $this->dashboardService->saveDefaultOrgDashboardsAndIndicators($dashboardsId,$additionalIndicators);
				$response->addDataSet(new DataSet('dashboards', $importedItems));
				$response->addDataSet(new DataSet('indicator', $indicatorImported));
				$response->addDataSet(new DataSet('dataSource', $dataSourceImported));
				$this->dashboardService->setLogImport($importedItems,$indicatorImported,$dataSourceImported);
				$nrImportedDashboards = count($importedItems);
				$response->addNotification(new DTO\Response\Notification("{$nrImportedDashboards} dashboard(s) persisted with success"));
			} else {
				$response->setError(new DTO\Response\Error("Invalid parameters. Missing the dashboards", 500));
			}
		} catch (\Exception $e) {
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

}